//---------------------------------------------------------------------------
#define WIN32

#define GRAFICO
//#undef GRAFICO

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <time.h>


#include "Desenha.h"

#define MAX(x,y) ((x)<(y) ? (y) : (x))

#pragma hdrstop

struct lista {
	int cidade_origem;
	int cidade_origem_i;
     int cidade_destino;
     int cidade_destino_j;
  	lista *proximo;  /* ponteiro para a prxima entrada */
  	lista *anterior; /* ponteiro para o registro anterior */
};

struct lista_restrita {
	int cidade_origem;
     int cidade_destino;
     double distancia;
     int fo;
  	lista_restrita *proximo;  /* ponteiro para a prxima entrada */
  	lista_restrita *anterior; /* ponteiro para o registro anterior */
};

struct vizinho{
        int nome;
        int distancia;
        vizinho *proximo;
        vizinho *anterior;
};

struct cidade {
        int nome;
        vizinho *vizinhos;
};

cidade *cria_vetor_cidade(int tam);

int *cria_vetor(int tam);

double **cria_matriz(int nlinhas, int ncolunas);

void libera_vetor(int *vetor);

void libera_vetor_cidades (cidade *cid, int n);

void libera_matriz(double **matriz, int nlinhas);

void le_arq_cidades(char nomearq[], cidade *cidades);

void le_arq_cidades_matriz(char nomearq[], int n, int *vet_x, int *vet_y, double **distancias);

void constroi_solucao_aleatoria(int *s, int num_cidades);

void troca_posicao(int *caixeiro, int x, int y);

void imprime_solucao(int *s, int num_objetos);

int calcula_fo(int *caxeiro, double **distancias, int n);

bool esta_na_solucao(int i, int *s, int n);

void troca_bit(int *s, int j);

double randomico(double min, double max);

void atualiza_melhor_solucao(int *s, int *s_star, int num_objetos);

void imprime_solucao(int *s, int num_objetos);

void inicializa_vetor(int *vetor, int tam, int valor);

void inicializa_vetor_cidades(cidade *cid, int n);

bool verifica_vizinho(int *caixeiro, cidade *cid, int n);

int melhor_vizinho(int *s, int num_objetos, int fo, int fo_star, int *melhor_i, int *melhor_j,
                   double **distancias, lista **iniciotabu, lista **finaltabu);

void apaga_registro_tabu(struct lista *registro, struct lista **iniciotabu, struct lista **finaltabu);

void apaga_lista_tabu(struct lista **iniciotabu, struct lista **finaltabu);

int lenght_lista_tabu(struct lista **iniciotabu, /* primeiro elemento da lista */
                      struct lista **finaltabu);  /* ltimo elemento da lista */

void display_lista_tabu(struct lista **iniciotabu,/* primeiro elemento da lista */
                        struct lista **finaltabu);  /* ltimo elemento da lista */

void insere_lista_tabu(struct lista *i, struct lista **iniciotabu, struct lista **finaltabu);

bool esta_lista_tabu(int cid_1, int i,int cid_2, int j, struct lista **iniciotabu);

void imprime_fo(char nomearq[], float tempo, int fo, int iteracao);

void limpa_arquivo(char nomearq[]);

void BT(int n, int *s, double **distancias, int *fo, int tamanho_maximo_lista_tabu, int BTmax, Desenha *desenho);

int menu_solucao_inicial(void);

int menu_principal(void);

void embaralha_vetor(int *vetor, int n);

void constroi_solucao_parcialmente_gulosa(int n, int *s, int b, double **distancias, int *fo, double alfa_GRASP);

void insere_LC(struct lista_restrita *i, struct lista_restrita **inicio_LC, struct lista_restrita **final_LC);

int lenght_LC(struct lista_restrita **inicio_LC, struct lista_restrita **final_LC);

struct lista_restrita *encontra_objeto_LC(int posicao, struct lista_restrita **inicio_LC);

void apaga_registro_LC(struct lista_restrita *registro, struct lista_restrita **inicio_LC, struct lista_restrita **final_LC);

void display_LC(struct lista_restrita **inicio_LC, struct lista_restrita **final_LC);

//---------------------------------------------------------------------------

#pragma argsused
int main(int argc, char **argv)
{
     /* Variveis do Sistema */
     int *s;                         // vetor solucao corrente
     int *s_star;                    // vetor melhor solucao
     int fo;                         // funcao objetivo corrente
     double **distancias;            // matriz com as distancias entre as cidades
     int choice;
     int tamanho_maximo_lista_tabu; // tamanho maximo da Lista Tabu
     int BTmax;
     Desenha *desenho;                     // numero max de iteracoes s/melhora da Busca Tabu
     int *vet_x, *vet_y;
     double alfa_GRASP;
     int b;

     /************ Nmero de Cidades ***********/
     int n = 224;
//     printf("Digite o nmero de cidades a ser visitadas [ <= 1000 ]: ");
//     scanf("%d",&n);
//     printf("%d",n);
     BTmax = 500;
     char *arquivo = "ts225.txt";                  // arquivo das cidades
     /******************************************/


     //srand(1000); // pega o numero 1000 como semente de numeros aleatorios
     srand((unsigned) time(NULL)); // pega a hora do relogio como semente

     /* Parametros para o GRASP */
     alfa_GRASP = 0.0;
     b = 100000;

     fo = 0;
     s = cria_vetor(n);
     s_star = cria_vetor(n);
     distancias = cria_matriz(n,n);
     vet_x = cria_vetor(n);
     vet_y = cria_vetor(n);

     inicializa_vetor(s,n,0);
     inicializa_vetor(s_star,n,0);

//  arquivo = "cidades" + n + ".txt";
//  printf("\n%s\n",arquivo);

     le_arq_cidades_matriz(arquivo,n,vet_x,vet_y,distancias);

   /* Gerao da soluo inicial */
     switch(menu_solucao_inicial()){
          case 1:
               constroi_solucao_parcialmente_gulosa(n,s,b,distancias,&fo,alfa_GRASP);
               printf("Solucao GRASP:\n");
               break;
          case 2:
               constroi_solucao_aleatoria(s, n);
               printf("Solucao aleatoria:\n");
               break;
     }



     imprime_solucao(s,n);

     fo = calcula_fo(s, distancias, n);
     printf("funcao objetivo = %3d \n",fo);

     printf("\n\nPressione <enter> para continuar...");


#ifdef GRAFICO

     desenho = new Desenha(false,argc,argv,s,vet_x,vet_y,n);

#endif


     getchar();getchar();


     /*************************************************************************/

     /**************************** Busca Tabu *********************************/

     /*************************************************************************/


     tamanho_maximo_lista_tabu = n * 0.10;

     BT (n, s, distancias, &fo, tamanho_maximo_lista_tabu, BTmax, desenho);


     /*************************************************************************/


     libera_vetor(s);
     libera_vetor(s_star);
     libera_vetor(vet_x);
     libera_vetor(vet_y);
//  libera_vetor_cidades(cidades,n);
     return 0;
} /* fim do programa principal */


//---------------------------------------------------------------------------


/* cria memoria para um vetor de tam posicoes */
cidade *cria_vetor_cidade(int tam){
  cidade *vetor;

  vetor = (cidade *) malloc(tam*sizeof(cidade));
  if (!vetor){
    printf("Falta memoria para alocar o vetor de ponteiros");
    exit(1);
  }
  return vetor;
}


/* cria memoria para um vetor de tam posicoes */
int *cria_vetor(int tam){
  int *vetor;

  vetor = (int *) malloc(tam*sizeof(int));
  if (!vetor){
    printf("Falta memoria para alocar o vetor de ponteiros");
    exit(1);
  }
  return vetor;
}

double **cria_matriz(int nlinhas, int ncolunas)
{
  register int i;
  double **matriz;

  matriz = (double **) malloc(nlinhas*sizeof(double *));
  if (!matriz) {
        printf("Falta memoria para alocar a matriz de ponteiros\n");
        exit(1);
  }
  for (i=0; i < nlinhas; i++) {
    matriz[i] = (double *) malloc(ncolunas*sizeof(double));
    if (!matriz[i]){
      printf("Falta memoria para alocar a matriz de ponteiros.\n");
      exit(1);
    }
  }
  return matriz;
}


void libera_matriz(double **matriz, int nlinhas)

{
  register int i;

  for (i=nlinhas-1; i >= 0; i--)
    free((int *) matriz[i]);
  free((int *) matriz);
}


/* libera memoria de um vetor */
void libera_vetor(int *vetor)
{
  free(vetor);
}

void libera_vetor_cidades (cidade *cid, int n){
     vizinho *temp;
     for (int i = 0; i < n; i++){
          if (cid[i].vizinhos != NULL){
               temp = cid[i].vizinhos;
               do{
                    temp = temp->proximo;
               }while(temp->proximo != NULL);
               while(temp->anterior != NULL){
                    temp = temp->anterior;
                    free(temp->proximo);
               }
               free(temp);
          }
     }
}


/* le um arquivo no formato cidade vizinho distancia */
void le_arq_cidades(char nomearq[], cidade *cidades)
{
  int cid, vizi, dist;
  vizinho* temp;
  FILE *arquivo;

//  printf("arquivo de cidades: %s\n",nomearq);
  arquivo = fopen(nomearq,"r");
  if (!arquivo) {
     printf("O Arquivo %s nao pode ser aberto.\n", nomearq);
     getchar();
     exit(1);
  }
  while (!feof(arquivo)){
    fscanf(arquivo, "%d %d %d", &cid, &vizi, &dist);
    temp = (vizinho *) malloc(sizeof(vizinho));
    if (!temp){
        printf("Falta memria para alocar cidades...\n");
        getchar();
        exit(1);
    }
    temp->nome = vizi;
    temp->distancia = dist;
    temp->proximo = cidades[cid].vizinhos;
    if (temp->proximo != NULL)
         temp->proximo->anterior = temp;
    temp->anterior = NULL;
    cidades[cid].vizinhos = temp;
  }
  fclose(arquivo);
}

void le_arq_cidades_matriz(char nomearq[], int n, int *vet_x, int *vet_y, double **distancias)
{
  int x, y, i, j;
//  double **distancia;
  FILE *arquivo;
  vizinho *temp;

//  distancia = cria_matriz(n,n);
  arquivo = fopen(nomearq,"r");
  if (!arquivo) {
     printf("O Arquivo %s nao pode ser aberto.\n", nomearq);
     getchar();
     exit(1);
  }
  for(int m=0; m < n; m++){
  //while (!feof(arquivo)){
    fscanf(arquivo, "%d %d %d", &i, &x, &y);
    vet_x[i] = x;
    vet_y[i] = y;
  }


  if (i+1 != n) { // verifica a "integridade" dos dados lidos
     printf("O Arquivo %s nao corresponde as especificacoes.\n", nomearq);
     printf("O Arquivo %s possui %d regitros enquanto o problema tem %d cidades.\n", nomearq, i, n);
     getchar();  exit(1);
  }
  // gera a matriz de distancias calculado a partir das distancias euclidianas dos pontos
  for(i=0; i < n; i++)
   {
     distancias[i][i] = 0;
     for(j=i; j < n; j++)
      {
        distancias[i][j] = (double) sqrt(pow(vet_x[i]-vet_x[j],2)+pow(vet_y[i]-vet_y[j],2));
        distancias[j][i] = distancias[i][j];
      }
   }

  fclose(arquivo);
}


/* constroi uma solucao inicial aleatoria */
void constroi_solucao_aleatoria(int *s, int num_cidades){
     for(int i = 0; i < num_cidades; i++){
          s[i] = i;
     }
     embaralha_vetor(s,num_cidades);
}


/* imprime a solucao */
void imprime_solucao(int *s, int num_objetos)
{
    for (int j=0; j<num_objetos; j++) printf(" %d ",s[j]);
    printf("\n");
}


/* calcula a funcao objetivo */
int calcula_fo(int *s, double **distancias, int n)
{
    int fo = 0;
    vizinho *temp;

    for (int i = 0; i < n - 1; i++){
          fo += distancias[s[i]][s[i+1]];
    }
    return fo;
}

bool esta_na_solucao(int i, int *s, int n){
     for (int j = 0; j < n; j++)
          if (s[j] == i) return true;
     return false;
}


/* insere ou retira o objeto j da mochila */
/*void troca_bit(int *s, int j)
{
    if (s[j])
      s[j] = 0;
    else
      s[j] = 1;
}
  */
void troca_posicao(int *s, int x, int y){
     int t = s[x];
     s[x] = s[y];
     s[y] = t;
}

/* Gera numero aleatorio entre min e max */
double randomico(double min, double max)
{
  if (min == max) return min;
  return ((double) (rand()%10000/10000.0)*(max-min) + min);
}

/* atualiza a melhor solucao */
void atualiza_melhor_solucao(int *s, int *s_star, int num_objetos)
{
   for (int j=0; j<num_objetos; j++) s_star[j] = s[j];
}

void inicializa_vetor(int *vetor, int tam, int valor)
{
    for (int j=0; j<tam; j++) vetor[j] = valor;
}

void inicializa_vetor_cidades(cidade *cid, int n){
     for (int i = 0; i < n; i++){
          cid[i].nome = i;
          cid[i].vizinhos = NULL;
     }
}

bool verifica_vizinho(int *caixeiro, cidade *cid, int n){
     bool ok = false;
     vizinho *temp;
     for (int i = 0; i < n - 1; i++){
          temp = cid[caixeiro[i]].vizinhos;
          ok = false;
          while(temp != NULL){
               if (caixeiro[i+1] == temp->nome){
                    ok = true;
                    break;
               }
               temp = temp->proximo;
          }
          if(!ok) return false;
     }
     return true;
}

int melhor_vizinho(int *s, int num_objetos, int fo, int fo_star, int *melhor_i, int *melhor_j,
                   double **distancias, lista **iniciotabu, lista **finaltabu)
{
    int delta, delta_max = INT_MAX;

    for (int i = 0; i < num_objetos; i++){
       for (int j = 0; j < num_objetos; j++){
          if (i != j){
               troca_posicao(s,i,j);
               int fo_cor = calcula_fo(s,distancias,num_objetos);
               delta = fo_cor - fo;
//               printf("delta = %d\n", delta);

               bool tabu = esta_lista_tabu(s[j],i,s[i],j,iniciotabu);
//               if (tabu == true) { printf("%d\n", tabu);getchar(); }
               if (!tabu || (tabu && (fo_cor <= fo_star))){
                    if(delta < delta_max){
                         delta_max = delta;
                         *melhor_i = i;
                         *melhor_j = j;
                    }
                    else if (delta == delta_max){
                         if (random(10) < 2){
                              delta_max = delta;
                              *melhor_i = i;
                              *melhor_j = j;
                         }
                    }
               troca_posicao(s,j,i);
               }
          }
       }
    }
    return delta_max;
}



/* Remove o elemento registro da lista */
void apaga_registro_tabu(struct lista *registro,
                         struct lista **iniciotabu,
                         struct lista **finaltabu)
{
  if(registro) {
    if(*iniciotabu==registro) {
      *iniciotabu=registro->proximo;
      if(*iniciotabu) (*iniciotabu)->anterior = NULL;
      else *finaltabu = NULL;
    }
    else {
      registro->anterior->proximo = registro->proximo;
      if(registro!=*finaltabu)
          registro->proximo->anterior = registro->anterior;
      else
        *finaltabu = registro->anterior;
    }
    free(registro); /* devolve memria para o sistema */
  }
}


/* Apaga TODA a lista tabu */
void apaga_lista_tabu(struct lista **iniciotabu,
                      struct lista **finaltabu)
{
    struct lista *registro = *iniciotabu;
    while(registro) {
	apaga_registro_tabu(registro, iniciotabu, finaltabu);
    	if (registro) registro = registro->proximo;   /* obtem prximo endereo */
    }
}


/* Mostra o comprimento da lista completa */
int lenght_lista_tabu(struct lista **iniciotabu, /* primeiro elemento da lista */
    	              struct lista **finaltabu)  /* ltimo elemento da lista */
{
  struct lista *registro = *iniciotabu;
  int cont = 0;

  while(registro) {
    registro = registro->proximo;   /* obtem prximo endereo */
    cont++;
  }
  return cont;
}


/* Mostra o conteudo da lista completa */
void display_lista_tabu(struct lista **iniciotabu, /* primeiro elemento da lista */
			struct lista **finaltabu)  /* ltimo elemento da lista */
{
  struct lista *registro = *iniciotabu;
  int cont = 0;

  while(registro) {
    printf("tabu[%2d]   Cidade Origem = %2d   Cidade Destino = %2d\n",
            cont, registro->cidade_origem, registro->cidade_destino);
    registro = registro->proximo;   /* obtem prximo endereo */
    cont++;
  }
}


/* Insere o registro i no final de uma lista duplamente encadeada */
void insere_lista_tabu(struct lista *i,
                       struct lista **iniciotabu,
		       struct lista **finaltabu)
{
  struct lista *antigo, *ponteiro;

  if (*finaltabu == NULL) {
    i->proximo  = NULL;
    i->anterior = NULL;
    *finaltabu = i;
    *iniciotabu = i;
    return;
  }
  ponteiro = *iniciotabu;
  antigo = NULL;
  while (ponteiro) {
    antigo = ponteiro;
    ponteiro = ponteiro->proximo;
  }
  antigo->proximo = i;
  i->proximo = NULL;
  i->anterior = antigo;
  *finaltabu = i;
}


/* Verifica se posicao esta na lista tabu */
bool esta_lista_tabu(int cid_1, int i, int cid_2, int j, struct lista **iniciotabu)
{
  struct lista *registro = *iniciotabu;

  while(registro != NULL) {
    if (registro->cidade_origem == cid_1 && registro->cidade_origem_i == i && registro->cidade_destino == cid_2 && registro->cidade_destino_j == j)
        return true;
    registro = registro->proximo;  /* obtem novo registro */
  }
  return false; /* nao encontrou */
}


void imprime_fo(char nomearq[], float tempo, int fo, int iteracao)
{
    FILE *arquivo;
    arquivo = fopen(nomearq,"a+");

    if (!arquivo){
        printf("O arquivo %s nao pode ser aberto",nomearq);
        getchar();
        exit(1);
    }
    fprintf(arquivo,"%4.2f\t  %4d\t  %5d\n",tempo, iteracao, fo);
    fclose(arquivo);
}


void limpa_arquivo(char nomearq[])
{
  FILE *arquivo;
  arquivo = fopen(nomearq,"w");
  if (!arquivo){
     printf("O arquivo %s nao pode ser aberto",nomearq);
     getchar();
     exit(1);
  }
  fclose(arquivo);
}


void BT(int n, int *s, double **distancias, int *fo, int tamanho_maximo_lista_tabu, int BTmax, Desenha *desenho)
{
    int melhor_i, melhor_j;
    struct lista *iniciotabu, *finaltabu, *tabu;
    int iterBT = 0;  // numero corrente de iteracoes da Busca Tabu
    int MelhorIter = 0; // Iteracao em que ocorreu a melhor solucao
    int *s_star;
    int fo_star;
    int delta;
    clock_t CPUiniciotabu, CPUfimtabu;

    s_star = cria_vetor(n);
    inicializa_vetor(s_star,n,0);
    atualiza_melhor_solucao(s, s_star, n);
    fo_star = *fo;

    printf("\nIniciando a Busca Tabu com fo = %d\n", *fo);
//    getchar();
    iniciotabu = finaltabu = NULL;
    CPUiniciotabu = CPUfimtabu = clock();
    limpa_arquivo("valordefoBT.out");
    imprime_fo("valordefoBT.out", (float)(CPUfimtabu - CPUiniciotabu)/CLOCKS_PER_SEC, *fo, iterBT);


     while (iterBT - MelhorIter < BTmax) {
          iterBT++;
          delta = melhor_vizinho(s, n, *fo, fo_star,&melhor_i, &melhor_j, distancias, &iniciotabu, &finaltabu);
          printf("\ndelta do melhor vizinho = %d melhor i = %d melhor j = %d\n", delta, melhor_i, melhor_j);
          //getchar();
          tabu = (struct lista *)malloc(sizeof(struct lista));
          if (!tabu) {
               printf("Faltando memoria ...\n");
               exit(1);
          }
          /* vou inserir um registro na lista tabu */
          tabu->cidade_origem = s[melhor_j];
     	tabu->cidade_origem_i  = melhor_j;
          tabu->cidade_destino = s[melhor_i];
          tabu->cidade_destino_j = melhor_i;


          if(!esta_lista_tabu(s[melhor_j],melhor_j,s[melhor_i],melhor_i,&iniciotabu))
         	     insere_lista_tabu(tabu, &iniciotabu, &finaltabu);

         /* vou calcular o tamanho da lista tabu e imprimi-la */
///	     printf("Tamanho da lista tabu = %2d\n", lenght_lista_tabu(&iniciotabu, &finaltabu));

//    	     display_lista_tabu(&iniciotabu, &finaltabu);

	   /* vou manter uma lista tabu com tamanho_maximo_lista_tabu elementos,
              isto e', se a lista tiver mais que essa quantidade de elementos,
              vou apagar o primeiro registro da lista */
	     if (lenght_lista_tabu(&iniciotabu, &finaltabu) > tamanho_maximo_lista_tabu){
          	/* vou apagar o primeiro registro da lista tabu */
               apaga_registro_tabu(iniciotabu, &iniciotabu, &finaltabu);
///               printf("novo Tamanho da lista tabu = %2d\n",lenght_lista_tabu(&iniciotabu, &finaltabu));
///               display_lista_tabu(&iniciotabu, &finaltabu);
         	}

	     troca_posicao(s,melhor_i,melhor_j);
          (*fo) += delta;
        printf("fo corrente = %d \n",*fo);
//        getchar();

          CPUfimtabu = clock();
          imprime_fo("valordefoBT.out", (float)(CPUfimtabu - CPUiniciotabu)/CLOCKS_PER_SEC, *fo, iterBT);
          if (*fo < fo_star){
               MelhorIter = iterBT;
	          atualiza_melhor_solucao(s, s_star, n);
               fo_star = *fo;
///	          imprime_solucao(s_star,n);
//               printf("\n****** fo_star = %3d **********\n",fo_star);
        }
#ifdef GRAFICO
        desenho->exibe();
#endif
//        getchar();
    }
    CPUfimtabu = clock();
    imprime_fo("valordefoBT.out", (float)(CPUfimtabu - CPUiniciotabu)/CLOCKS_PER_SEC, *fo, iterBT);
    printf("********** Melhor Solucao Obtida *************** \n");
    imprime_solucao(s_star,n);

//    *peso_objetos = calcula_peso_objetos(s_star, n, w);
//    *inv = calcula_inviabilidade(*peso_objetos, b);

    printf("Saindo da Busca Tabu com fo_star = %d\n",fo_star);
//    printf("peso dos objetos = %2d inviabilidade = %d \n",*peso_objetos, *inv);
    printf("Numero de solucoes analisadas = %ld \n",iterBT);
    printf("Numero de solucoes existentes = %.0f \n", exp(n*log(2)));
    printf("******************************************** \n");
    getchar();
    atualiza_melhor_solucao(s_star, s, n);
    libera_vetor(s_star);
    *fo = fo_star;
}

void embaralha_vetor(int *vetor, int n)
{
  int aux, j1, j2;

  for (int i=0; i < n; i++)
  {
    j1 = random(n);
    j2 = random(n);
    while (j1 == j2) j2 = random(n);
    aux = vetor[j1];
    vetor[j1] = vetor[j2];
    vetor[j2] = aux;
  }
}

void constroi_solucao_parcialmente_gulosa(int n, int *s, int b, double **distancias, int *fo, double alfa_GRASP)
{
    struct lista_restrita *inicio_LC, *final_LC, *candidato;
    int posicao_escolhida; // objeto escolhido
    int delta;   // variacao de energia
    int fo_star;
    int tamanho_LC, tamanho_LRC, cont;

    /* Inicio da Fase de Construcao de uma solucao */
    printf("\n Construindo nova solucao ...\n");
    inicio_LC = final_LC = NULL;
    inicializa_vetor(s,n,-1);
    *fo = 0;
    cont = 0;
    for (int i=0; i<n; i++){
       for (int j = i + 1; j < n; j++){

          candidato = (struct lista_restrita *)malloc(sizeof(struct lista_restrita));
          if (!candidato) {
               printf("Faltando memoria ...\n");
               exit(1);
          }
          /* vou inserir um registro em uma lista de candidatos,
               ordenada pelo beneficio relativo  */
          candidato->cidade_origem = i;
          candidato->cidade_destino = j;
          candidato->distancia = distancias[i][j];
          insere_LC(candidato, &inicio_LC, &final_LC);
          tamanho_LC = lenght_LC(&inicio_LC, &final_LC);
//          printf("Tamanho da LC = %2d\n", tamanho_LC);
//          display_LC(&inicio_LC, &final_LC);
//      getchar();
       }
    }
    display_LC(&inicio_LC, &final_LC);
    while (tamanho_LC > 0){
       tamanho_LRC = MAX(1,(int) (alfa_GRASP * tamanho_LC));
//       printf("Tamanho da LRC = %2d \n",tamanho_LRC);
       posicao_escolhida = random(tamanho_LRC);
//       printf("Posicao escolhida da LRC = %2d \n",posicao_escolhida);
       candidato = encontra_objeto_LC(posicao_escolhida, &inicio_LC);
//       printf("Objeto escolhido = %2d \n",candidato->objeto);
       if ( ((*fo + candidato->distancia) <= b) & (!esta_na_solucao(candidato->cidade_origem,s,n)) & (!esta_na_solucao(candidato->cidade_destino,s,n))){
         s[cont] = candidato->cidade_origem;
         s[cont+1] = candidato->cidade_destino;
         (*fo) += candidato->distancia;
         cont += 2;
//         printf("Objeto %2d de peso %2d incluido na solucao \n",candidato->objeto, w[candidato->objeto]);
//         printf("Peso atual dos objetos na mochila = %2d \n",*peso_objetos);
       }
//       else
//         printf("A inclusao do objeto %2d de peso %2d supera capacidade da mochila \n", candidato->objeto, w[candidato->objeto]);
//       printf("Vou eliminar o objeto %2d da LC\n",candidato->objeto);
//       getchar();
       apaga_registro_LC(candidato, &inicio_LC, &final_LC);
       tamanho_LC--;
//       display_LC(&inicio_LC, &final_LC);
    }
    /* Fim da Fase de Construcao de uma solucao */

    *fo = calcula_fo(s,distancias,n);
}

void insere_LC(struct lista_restrita *i,    /* novo elemento */
               struct lista_restrita **inicio_LC, /* primeiro elemento da lista */
               struct lista_restrita **final_LC)  /* ltimo elemento da lista */
{
  struct lista_restrita *antigo, *ponteiro;

  if (*final_LC==NULL) {  /* primeiro elemento da lista */
     i->proximo = NULL;
     i->anterior = NULL;
     *final_LC = i;
     *inicio_LC = i;
     return;
  }
  ponteiro = *inicio_LC; /* comea no topo da lista */
  antigo = NULL;
  while(ponteiro) {
    if(ponteiro->distancia < i->distancia) {
      antigo = ponteiro;
      ponteiro = ponteiro->proximo;
    }
    else {
      if(ponteiro->anterior) {
        ponteiro->anterior->proximo = i;
        i->proximo = ponteiro;
        i->anterior = ponteiro->anterior;
        ponteiro->anterior = i;
        return;
      }
      i->proximo = ponteiro; /* novo primeiro elemento */
      i->anterior = NULL;
      ponteiro->anterior = i;
      *inicio_LC = i;
      return;
    }
  }
  antigo->proximo = i; /* coloca no final */
  i->proximo = NULL;
  i->anterior = antigo;
  *final_LC = i;
}

int lenght_LC(struct lista_restrita **inicio_LC, /* primeiro elemento da lista */
              struct lista_restrita **final_LC)  /* ltimo elemento da lista */
{
  struct lista_restrita *registro = *inicio_LC;
  int cont = 0;

  while(registro) {
    registro = registro->proximo;   /* obtem prximo endereo */
    cont++;
  }
  return cont;
}

struct lista_restrita *encontra_objeto_LC(int posicao, struct lista_restrita **inicio_LC)
{
  struct lista_restrita *registro = *inicio_LC;
  int contador = 0;

  while (registro) {
    if (contador == posicao) return registro;
    contador++;
    registro = registro->proximo;  /* obtem novo registro */
  }
  printf("Registro nao encontrado.\n");
  return NULL; /* nao encontrou */
}

void apaga_registro_LC(struct lista_restrita *registro, struct lista_restrita **inicio_LC, struct lista_restrita **final_LC)
{
  if (registro) {
    if (*inicio_LC==registro) {
       *inicio_LC=registro->proximo;
       if (*inicio_LC)
          (*inicio_LC)->anterior = NULL;
       else *final_LC = NULL;
    }
    else {
      registro->anterior->proximo = registro->proximo;
      if (registro!=*final_LC)
         registro->proximo->anterior = registro->anterior;
      else
         *final_LC = registro->anterior;
    }
    free(registro); /* devolve memria para o sistema */
  }
}

int menu_solucao_inicial(void)
{
  int escolha;

  do {
    printf("\n*****************************************************");
    printf("\n          GERACAO DA SOLUCAO INICIAL: \n");
    printf("                  1. Parcialmente gulosa (GRASP) \n");
    printf("                  2. Aleatoria \n");
    printf("                  escolha: ");
    scanf("%d",&escolha);
  } while (escolha < 1 || escolha > 3);
  return escolha;
}

void display_LC(struct lista_restrita **inicio_LC, struct lista_restrita **final_LC)
{
  struct lista_restrita *registro = *inicio_LC;
  int cont = 0;

  while (registro) {
    printf("LC[%5d]   origem = %5d  destino = %5d  distancia = %5f \n",
            cont, registro->cidade_origem, registro->cidade_destino, registro->distancia);
    registro = registro->proximo;   /* obtem prximo endereo */
    cont++;
  }
  printf("\n");
}

